using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;
using UnityEngine.UI;
using TMPro;

public class BossAI : MonoBehaviour
{
  public bool battleStart = false;
  private GameObject player;
  public NavMeshAgent agent;
  public LayerMask whatIsPlayer, whatIsGround;

  public GameObject attackParticles;

  public Animator animator;

  public float waitTime = 2f;

  public float walkPointRange = 10f;
  public Vector3 walkPoint;
  bool walkPointSet;

  public AudioSource audioSource;
  public AudioClip audioClip;

  private ActionType actionState = ActionType.Idle;

  private ReceiveDamage playerDamage;
  private enum ActionType
  {
    Idle,
    Attack1,
    Attack2,
    Attack3
  }
  

  private bool isIdle = false;
  // Start is called before the first frame update
  void Start()
  {
    player = GameObject.Find("AmeliaDummy");
    playerDamage = player.GetComponent<ReceiveDamage>();
  }

  // Update is called once per frame
  void Update()
  {
    if (battleStart)
    {
      switch (actionState)
      {
        case ActionType.Idle:
          if (!isIdle) Idle();
          break;
        case ActionType.Attack1:
          Attack1();
          break;
        case ActionType.Attack2:
          Attack2();
          break;
        case ActionType.Attack3:
          Attack3();
          break;
      }
    }
  }

  private void Idle()
  {
    isIdle = true;
    Patroling();
    StartCoroutine(WaitForSomeTime(waitTime));
  }

  private void ChooseAction()
  {
    isIdle = false;
    Debug.Log("ChooseAction");
    int nextAction = Random.Range(1, 4);
    switch (nextAction)
    {
      case 1:
        actionState = ActionType.Idle;
        break;
      case 2:
        actionState = ActionType.Attack1;
        break;
      case 3:
        actionState = ActionType.Attack2;
        break;
      case 4:
        actionState = ActionType.Attack3;
        break;
    }
  }

  private void Patroling()
  {
    animator.SetTrigger("Move");
    if (!walkPointSet) SearchWalkPoint();

    if (walkPointSet)
      agent.SetDestination(walkPoint);

    Vector3 distanceToWalkPoint = transform.position - walkPoint;

    //Walkpoint reached
    if (distanceToWalkPoint.magnitude < 1f)
      walkPointSet = false;
  }

  private void SearchWalkPoint()
  {
    //Calculate random point in range
    float randomZ = Random.Range(-walkPointRange, walkPointRange);
    float randomX = Random.Range(-walkPointRange, walkPointRange);

    walkPoint = new Vector3(transform.position.x + randomX, transform.position.y, transform.position.z + randomZ);

    if (Physics.Raycast(walkPoint, -transform.up, 2f, whatIsGround))
      walkPointSet = true;

    StartCoroutine(WaitForSomeTime(waitTime));
  }

  private void Attack1()
  {
    agent.SetDestination(player.transform.position);
    if (Vector3.Distance(transform.position, player.transform.position) < 3)
    {
      //Do the animation
      bool playerInSightRange = Physics.CheckSphere(transform.position, 7, whatIsPlayer);
      if (playerInSightRange)
      {
        animator.SetTrigger("Attack1");
        playerDamage.ReduceHealth(2);
        Debug.Log("Attack1");
        MakeEffect();
        audioSource.clip = audioClip;
        audioSource.Play();
      }
      actionState = ActionType.Idle;
      StartCoroutine(WaitForSomeTime(waitTime));

      //land
    }
  }

  private void Attack2()
  {
    agent.SetDestination(player.transform.position);
    if (Vector3.Distance(transform.position, player.transform.position) < 4)
    {
      //Do the animation
      bool playerInSightRange = Physics.CheckSphere(transform.position, 7, whatIsPlayer);
      if (playerInSightRange)
      {
        animator.SetTrigger("Attack2");
        playerDamage.ReduceHealth(3);
        Debug.Log("Attack2");
        MakeEffect();
        audioSource.clip = audioClip;
        audioSource.Play();
      }
      actionState = ActionType.Idle;
      StartCoroutine(WaitForSomeTime(waitTime));

      //land
    }
  }

  private void Attack3()
  {
    agent.SetDestination(player.transform.position);
    if (Vector3.Distance(transform.position, player.transform.position) < 5)
    {
      //Do the animation
      bool playerInSightRange = Physics.CheckSphere(transform.position, 10, whatIsPlayer);
      if (playerInSightRange)
      {
        animator.SetTrigger("Attack3");
        playerDamage.ReduceHealth(5);
        Debug.Log("Attack3");
        MakeEffect();
        audioSource.clip = audioClip;
        audioSource.Play();
      }
      actionState = ActionType.Idle;
      StartCoroutine(WaitForSomeTime(waitTime));

      //land
    }
  }

  private void MakeEffect()
  {
    Instantiate(attackParticles, new Vector3(player.transform.position.x, 1.9f, player.transform.position.z), Quaternion.identity);
  }

  IEnumerator WaitForSomeTime(float seconds)
  {
    Debug.Log("startWait");
    yield return new WaitForSeconds(seconds);
    ChooseAction();
  }
}
