
using UnityEngine;
using UnityEngine.AI;

public class EnemyAI : MonoBehaviour
{
  public NavMeshAgent agent;

  private Transform player;

  public LayerMask whatIsGround, whatIsPlayer;

  public float health;

  public CheckForDamage attackScript;
  public float targetHeight = 1.45f;

  //Patroling
  public Vector3 walkPoint;
  bool walkPointSet;
  public float walkPointRange;

  //Attacking
  public float timeBetweenAttacks;
  bool alreadyAttacked;
  public GameObject projectile;

  //States
  public float sightRange, attackRange;
  public bool playerInSightRange, playerInAttackRange;

  public Animator animator;
  public AudioSource source;
  public AudioClip clip;

  private void Start()
  {
    player = GameObject.Find("AmeliaDummy").transform;
    agent = GetComponent<NavMeshAgent>();
  }

  private void Update()
  {
    //Check for sight and attack range
    playerInSightRange = Physics.CheckSphere(transform.position, sightRange, whatIsPlayer);
    playerInAttackRange = Physics.CheckSphere(transform.position, attackRange, whatIsPlayer);

    if (!playerInSightRange && !playerInAttackRange) Patroling();
    if (playerInSightRange && !playerInAttackRange) ChasePlayer();
    if (playerInAttackRange && playerInSightRange) AttackPlayer();
  }

  private void Patroling()
  {
    animator.SetTrigger("Idle");
    if (!walkPointSet) SearchWalkPoint();

    if (walkPointSet)
      agent.SetDestination(walkPoint);

    Vector3 distanceToWalkPoint = transform.position - walkPoint;

    //Walkpoint reached
    if (distanceToWalkPoint.magnitude < 1f)
      walkPointSet = false;
  }
  private void SearchWalkPoint()
  {
    //Calculate random point in range
    float randomZ = Random.Range(-walkPointRange, walkPointRange);
    float randomX = Random.Range(-walkPointRange, walkPointRange);

    walkPoint = new Vector3(transform.position.x + randomX, transform.position.y, transform.position.z + randomZ);

    if (Physics.Raycast(walkPoint, -transform.up, 2f, whatIsGround))
      walkPointSet = true;
  }

  private void ChasePlayer()
  {
    agent.SetDestination(player.position);
  }

  private void AttackPlayer()
  {
    //Make sure enemy doesn't move
    agent.SetDestination(transform.position);

    transform.LookAt(new Vector3(player.position.x, targetHeight, player.position.z));

    if (!alreadyAttacked)
    {
      attackScript.attackRequired = true;
      animator.SetTrigger("Attack");
      source.clip = clip;
      source.Play();
      alreadyAttacked = true;
      Invoke(nameof(ResetAttack), timeBetweenAttacks);
    }
  }
  private void ResetAttack()
  {
    alreadyAttacked = false;
  }

  public void TakeDamage(int damage)
  {
    health -= damage;

    if (health <= 0) Invoke(nameof(DestroyEnemy), 0.5f);
  }
  private void DestroyEnemy()
  {
    Destroy(gameObject);
  }

  private void OnDrawGizmosSelected()
  {
    Gizmos.color = Color.red;
    Gizmos.DrawWireSphere(transform.position, attackRange);
    Gizmos.color = Color.yellow;
    Gizmos.DrawWireSphere(transform.position, sightRange);
  }
}
