using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

public class GameManager : MonoBehaviour
{
  static GameManager _instance;

  public static GameManager managerInstance
  {
    get
    {
      if (_instance == null)
      {
        _instance = GameObject.FindGameObjectWithTag("Manager").GetComponent<GameManager>();
        //Tell unity not to destroy this object when loading a new scene!
        DontDestroyOnLoad(_instance.gameObject);
      }
      return _instance;
    }
  }

  void Awake()
  {
    if (_instance == null)
    {
      //If I am the first instance, make me the Singleton
      _instance = this;
      DontDestroyOnLoad(this);
    }
    else
    {
      //If a Singleton already exists and you find
      //another reference in scene, destroy it!
      if (this != _instance)
      {
        Destroy(this.gameObject);
      }
    }
  }

  private void Start()
  {
    Cursor.lockState = CursorLockMode.Locked;
    SceneManager.LoadSceneAsync("UI", LoadSceneMode.Additive);
    SceneManager.LoadSceneAsync("Base", LoadSceneMode.Additive);
    SceneManager.LoadSceneAsync("FirstLevel", LoadSceneMode.Additive);
  }

  public void QuitGame()
  {
    Application.Quit();
  }
}
